@@ -8,19 +8,20 @@ import copy
 import json
 import os
 import pathlib
+import re
 import sys
 import sysconfig
 from typing import Any, Sequence, cast
 
 from lsprotocol.types import (
     CODE_ACTION_RESOLVE,
-    EXIT,
     INITIALIZE,
     TEXT_DOCUMENT_CODE_ACTION,
     TEXT_DOCUMENT_DID_CHANGE,
     TEXT_DOCUMENT_DID_CLOSE,
     TEXT_DOCUMENT_DID_OPEN,
     TEXT_DOCUMENT_DID_SAVE,
+    TEXT_DOCUMENT_HOVER,
     AnnotatedTextEdit,
     CodeAction,
     CodeActionKind,
@@ -33,7 +34,11 @@ from lsprotocol.types import (
     DidCloseTextDocumentParams,
     DidOpenTextDocumentParams,
     DidSaveTextDocumentParams,
+    Hover,
+    HoverParams,
     InitializeParams,
+    MarkupContent,
+    MarkupKind,
     MessageType,
     OptionalVersionedTextDocumentIdentifier,
     Position,
@@ -46,15 +51,15 @@ from lsprotocol.types import (
 from pygls import protocol, server, uris, workspace
 from typing_extensions import TypedDict
 
-from ruff_lsp import jsonrpc, utils
+from ruff_lsp import utils
 
 WORKSPACE_SETTINGS: dict[str, dict[str, Any]] = {}
-RUNNER = pathlib.Path(__file__).parent / "runner.py"
+INTERPRETER_PATHS: dict[str, str] = {}
 
 MAX_WORKERS = 5
 LSP_SERVER = server.LanguageServer(
     name="Ruff",
-    version="2022.0.22",
+    version="2022.0.23",
     max_workers=MAX_WORKERS,
 )
 
@@ -182,6 +187,41 @@ def _get_severity(*_codes: list[str]) -> DiagnosticSeverity:
     return DiagnosticSeverity.Warning
 
 
+NOQA_REGEX = re.compile(r"(?i:# noqa)(?::\s?(?P<codes>([A-Z]+[0-9]+(?:[,\s]+)?)+))?")
+CODE_REGEX = re.compile(r"[A-Z]{1,3}[0-9]{3}")
+
+
+@LSP_SERVER.feature(TEXT_DOCUMENT_HOVER)
+def hover(params: HoverParams) -> Hover | None:
+    """LSP handler for textDocument/hover request."""
+    document = LSP_SERVER.workspace.get_document(params.text_document.uri)
+    match = NOQA_REGEX.search(document.lines[params.position.line])
+    if not match:
+        return None
+
+    codes = match.group("codes")
+    if not codes:
+        return None
+
+    codes_start = match.start("codes")
+    for match in CODE_REGEX.finditer(codes):
+        start, end = match.span()
+        start += codes_start
+        end += codes_start
+        if start <= params.position.character < end:
+            code = match.group()
+            result = _run_subcommand_on_document(document, ["--explain", code])
+            if result.stdout:
+                return Hover(
+                    contents=MarkupContent(
+                        kind=MarkupKind.Markdown,
+                        value=result.stdout.strip(),
+                    )
+                )
+
+    return None
+
+
 # **********************************************************
 # Linting features end here
 # **********************************************************
@@ -474,12 +514,6 @@ def initialize(params: InitializeParams) -> None:
             LSP_SERVER.lsp.trace = TraceValues.Off
 
 
-@LSP_SERVER.feature(EXIT)
-def on_exit():
-    """Handle clean up on exit."""
-    jsonrpc.shutdown_json_rpc()
-
-
 # *****************************************************
 # Internal functional and settings management APIs.
 # *****************************************************
@@ -536,6 +570,28 @@ def _get_settings_by_document(document: workspace.Document | None) -> dict[str,
 # *****************************************************
 # Internal execution APIs.
 # *****************************************************
+def _executable_path(settings: dict[str, Any]) -> list[str]:
+    """Returns the path to the executable."""
+    if settings["path"]:
+        # 'path' setting takes priority over everything.
+        return settings["path"]
+    elif settings["interpreter"] and not utils.is_current_interpreter(
+        settings["interpreter"][0]
+    ):
+        # If there is a different interpreter set, find its scripts path.
+        if settings["interpreter"][0] not in INTERPRETER_PATHS:
+            INTERPRETER_PATHS[settings["interpreter"][0]] = utils.scripts(
+                settings["interpreter"][0]
+            )
+        return [
+            os.path.join(INTERPRETER_PATHS[settings["interpreter"][0]], TOOL_MODULE)
+        ]
+    else:
+        # If the interpreter is same as the interpreter running this process, get the
+        # scripts path directly.
+        return [os.path.join(sysconfig.get_path("scripts"), TOOL_MODULE)]
+
+
 def _run_tool_on_document(
     document: workspace.Document,
     use_stdin: bool = False,
@@ -554,75 +610,46 @@ def _run_tool_on_document(
         log_warning(f"Skipping standard library file: {document.path}")
         return None
 
-    # deep copy here to prevent accidentally updating global settings.
+    # Deep copy, to prevent accidentally updating global settings.
     settings = copy.deepcopy(_get_settings_by_document(document))
 
-    code_workspace = settings["workspaceFS"]
-    cwd = settings["workspaceFS"]
-
-    use_path = False
-    use_rpc = False
-    if settings["path"]:
-        # 'path' setting takes priority over everything.
-        use_path = True
-        argv = settings["path"]
-    elif settings["interpreter"] and not utils.is_current_interpreter(
-        settings["interpreter"][0]
-    ):
-        # If there is a different interpreter set use JSON-RPC to the subprocess
-        # running under that interpreter.
-        argv = [TOOL_MODULE]
-        use_rpc = True
-    else:
-        # If the interpreter is same as the interpreter running this process then run
-        # as module.
-        argv = [os.path.join(sysconfig.get_path("scripts"), TOOL_MODULE)]
-
-    argv += TOOL_ARGS + settings["args"] + list(extra_args)
-
+    argv = _executable_path(settings) + TOOL_ARGS + settings["args"] + list(extra_args)
     if use_stdin:
         argv += ["--stdin-filename", document.path]
     else:
         argv += [document.path]
 
-    result: utils.RunResult
-    if use_path:
-        # This mode is used when running executables.
-        result = utils.run_path(
-            argv=argv,
-            use_stdin=use_stdin,
-            cwd=cwd,
-            source=document.source.replace("\r\n", "\n"),
-        )
-        if result.stderr:
-            log_to_output(result.stderr)
-    elif use_rpc:
-        # This mode is used if the interpreter running this server is different from
-        # the interpreter used for running this server.
-        rpc_result = jsonrpc.run_over_json_rpc(
-            workspace=code_workspace,
-            interpreter=settings["interpreter"],
-            module=TOOL_MODULE,
-            argv=argv,
-            use_stdin=use_stdin,
-            cwd=cwd,
-            source=document.source,
-        )
-        if rpc_result.exception:
-            log_error(rpc_result.exception)
-        elif rpc_result.stderr:
-            log_to_output(rpc_result.stderr)
-        result = utils.RunResult(rpc_result.stdout, rpc_result.stderr)
-    else:
-        # This mode is used when running executables.
-        result = utils.run_path(
-            argv=argv,
-            use_stdin=use_stdin,
-            cwd=cwd,
-            source=document.source.replace("\r\n", "\n"),
-        )
-        if result.stderr:
-            log_to_output(result.stderr)
+    log_to_output(f"Running Ruff with: {argv}")
+    result: utils.RunResult = utils.run_path(
+        argv=argv,
+        use_stdin=use_stdin,
+        cwd=settings["workspaceFS"],
+        source=document.source.replace("\r\n", "\n"),
+    )
+    if result.stderr:
+        log_to_output(result.stderr)
+
+    return result
+
+
+def _run_subcommand_on_document(
+    document: workspace.Document,
+    args: Sequence[str],
+) -> utils.RunResult:
+    """Runs the tool subcommand on the given document."""
+    # Deep copy, to prevent accidentally updating global settings.
+    settings = copy.deepcopy(_get_settings_by_document(document))
+
+    argv = _executable_path(settings) + list(args)
+
+    log_to_output(f"Running Ruff with: {argv}")
+    result: utils.RunResult = utils.run_path(
+        argv=argv,
+        use_stdin=False,
+        cwd=settings["workspaceFS"],
+    )
+    if result.stderr:
+        log_to_output(result.stderr)
 
     return result
 
